fl

{
   2-wire Driver for an HD44780 based LCD Character based display   (8bit control)
   Used Hex D-F/F(74HC174)
   
    PropForth 5.2 
    17/09/2012 20:47:27

     2-wire LCD curcuit   Propeller
     latch                P0   0x0
     clk                  P1   0x1
     LCD backlight        P2   0x2      wire should be short 
}

hex


\ _cfo ( n1 -- n2 ) n1 - desired frequency, n2 frqa/frqb 
: _cfo clkfreq 1- min 0 swap clkfreq um/mod swap clkfreq 2/ >= abs + ; 

1F8	wconstant ctra 
1FA	wconstant frqa

\  Edit 2 constant below to match your hardware
0 wconstant _latch
1 wconstant _clk
\ Comment out when not using brightness-control
2 wconstant _bright

\ set LCD's braightness to oscillate pin at the specified frequency
\ ( n1 --)  n1 is the frequency, uses ctra  NCO-mode
: set_bright _cfo frqa COG! _bright dup pinout 10000000 + ctra COG! ; 


wvariable char           \ character number for LCD:x           
wvariable line           \ line number for LCD:y
wvariable cur_line       \ current line number

\ serial out data(6bit) to shift-register
\ ( n1 n2 n3 -- )  n1:shift-data n2:_latchm n3:_clkm
lockdict create a_shift_data_8 forthentry
$C_a_lxasm w, h124  h113  1- tuck - h9 lshift or here W@ alignl h10 lshift or l,
z2WiPZB l, z1SyLI[ l, z2WiPeB l, z1SyLI[ l, zfyPOM l, z2WyPjB l, zgyPO1 l, z1bfxZD l,
z1bixZC l, z2WiPyk l, z20yPrS l, z3ryPr0 l, z1[ixZD l, z1[ixZC l, z3[yPnP l, z1SyLI[ l,
z1SV01X l,
freedict

\ ( -- ) set shift-register output to zero(6bit)
lockdict create a_clk_out_8 forthentry
$C_a_lxasm w, h11C  h113  1- tuck - h9 lshift or here W@ alignl h10 lshift or l,
z2WyPWA l, z1bixZB l, 0 l, 0 l, z1[ixZB l, 0 l, z3[yP[K l, z1SyLI[ l,
z1SV01X l,
freedict

: _latch_out_l _latch pinlo ;
: _latch_out_h _latch pinhi ;
: _clk_out_l _clk pinlo ;
: _clk_out_h _clk pinhi ;

1 _latch lshift constant _latchm
1 _clk lshift constant _clkm

\ send hi-pulse to Enable-pin
: lcd_enable _latch_out_h _latch_out_l ;

\ send data to shift-register
\ ( n -- )   n:lcd4bit(bit0-3) + RS(bit4) + E(bit5)
: shift_out 
     \ ( n1 n2 n3 -- )  n1:shift-data n2:_latchm n3:_clkm
     _latchm _clkm a_shift_data_8     
;

\ reset shift-register
: reset_sr _clkm a_clk_out_8 ;
                                                          
\ lcd_com - send command to HD44780   (n -- )  n:bit8=RS-bit  bit7-0=LCD-data
: lcd_com                                                      
invert 1FF and dup 100 and if 100 else 0 then or 200 or 
shift_out
lcd_enable
reset_sr       
;

\ setup propeller pins and initialize HD44780
: lcd_init
\ set from P**(_data) to P**(_clk) to output
_latch 2 0 do dup pinout 1+ loop drop
reset_sr                

30 lcd_com                                    
5 delms 
lcd_enable                                  
1 delms                                      
lcd_enable                                   
5 delms
reset_sr                           \ reset shift-register

38 lcd_com    1 delms
8 lcd_com     1 delms
1 lcd_com     1 delms
6 lcd_com     1 delms
C lcd_com     1 delms

10 char W! 2 line W!               \ defsult setting is 16Charcters & 2Lines

3E8 set_bright                     \ 1kHz
;

\ display character  ( c -- )
: lcd_char
100 or                             \ Add RS=1 to character-code                                
lcd_com    
reset_sr                           \ delay time
;                                                                           

{
x -- horizontal pos  : 0x1 to 0x28 (decimal: 1 to 40Characters)
y -- line number     : 0x1 to 0x4  (decimal: 1 to 4Lines)
( x y -- )
}
: lcd_pos
2 u/mod swap 0= 
if 1 =     \ line is even?
     if 40 else char W@ 10 = if 50 else 54 then then
else 0=
     if 0 else char W@ 10 = if 10 else  14 then then 
then
1- + 80 or lcd_com 
;

\ clear lcd ( -- )
: lcd_clr 1 lcd_com 2 delms ;

\ setup of display construction  (x y -- )
: lcd_setup line W! char W! ;

\ display string  ( cstr -- )
: lcd_str C@++ dup if bounds do i C@ lcd_char loop else 2drop then ;

\ output cr
: lcd_cr cur_line W@ 1+ dup line W@ > if drop 1 1 lcd_pos 1 cur_line W! else dup 1 swap lcd_pos cur_line W! then ;

\ Display decimal-number from Left
\ (n -- )  n:decimal or hex
variable tmp
wvariable result     
: lcd_dec_L
0 result W! 3B9ACA00 tmp L!
dup 80000000 and if invert 1+ 2D lcd_char then         \ check minus?  
A 0 do 
     dup tmp L@ >= 
     if 
          tmp L@ u/mod 30 + lcd_char 1 result W! 
     else 
          result W@ tmp L@ 1 = or 
          if 30 lcd_char then
     then
     tmp L@ A u/ tmp L! 
loop
drop
;

\ Display decimal-number from Right
\ cur_line is used for displaying number
\ ( n -- )  n:decimal or hex
: lcd_dec_R
0 result W! 3B9ACA00 tmp L!
dup 80000000 and if invert 1+ 0 else 1 then swap         \ check minus?  
A 0 do 
     dup tmp L@ >= 
     if 
          result W@ 0=
          if
               swap 0= if i cur_line W@ lcd_pos 2D lcd_char then
          then
          tmp L@ u/mod 30 + lcd_char 1 result W! 
     else 
          result W@ tmp L@ 1 = or 
          if 30 lcd_char 
          else bl lcd_char
          then
     then
     tmp L@ A u/ tmp L! 
loop
drop
;

\ display hex number  (n1 n2 -- ) n1:hex-value  n2:digits(1 to 8)
: lcd_hex
dup rot2 8 swap - 2 lshift lshift swap
0 do 
     dup F0000000 and 1C rshift dup A < 
     if 30 + else 37 + then 
     lcd_char 
     4 lshift 
loop
drop
;
\ display binary number (n1 n2 -- ) n1:hex-value  n2:digits(1 to 10)
: lcd_bin
dup rot2 20 swap - lshift swap  
0 do 
     dup 80000000 and 
     if 31 else 30 then 
     lcd_char 
     1 lshift 
loop 
drop
;

\ output spaces  ( n -- ) n:space's number output to LCD 
: lcd_bl dup if 0 do bl lcd_char loop else drop then ;

: sleep        \ LCD Off (not erase display-data)
8 lcd_com      \ display OFF
0 ctra COG! _bright pinin
;

: wakeup       \ LCD On
C lcd_com      \ display ON
3E8 set_bright        \ 1kHz
;


decimal

{

fl
build_BootOpt :rasm
          \ get _clkm
     mov       $C_treg1 , $C_stTOS
     spop
          \ get _latchm
     mov       $C_treg2 , $C_stTOS
     spop
          \ shift data to left
     shl       $C_stTOS , # d22
          \ set loop-count  8bit-mode
     mov       $C_treg3 , # d11
__1
     shl       $C_stTOS , # 1 wc
          \ set latch to Hi/Lo
if_c or        outa , $C_treg2
          \ clock pulse Hi:500nsec  Lo:200nsec
     or        outa , $C_treg1
     mov       $C_treg4 , cnt
     add       $C_treg4 , # d28
     waitcnt   $C_treg4 , # 0
          \ set _latch to Lo    
     andn      outa , $C_treg2     
     andn      outa , $C_treg1
     
     djnz      $C_treg3 , # __1
          
     spop
     jexit
;asm a_shift_data_8

fl
build_BootOpt :rasm
          \ set loop-count  8bit-mode
     mov       $C_treg1 , # d10
__1
          \ pulse width Hi:150nsec  Lo:150nsec
     or        outa , $C_stTOS
     nop
     nop
     andn      outa , $C_stTOS
     nop
     djnz      $C_treg1 , # __1
     
     spop
     jexit     
;asm a_clk_out_8

}
{
fl

\ check clock-pulse on a_shift_data_8 by using oscilloscope
: test1 begin 55 shift_out fkey? swap drop until ;

\ check clock-pulse on a_clk_out_8 by using oscilloscope
: test2 begin reset_sr fkey? swap drop until ;


}