fl

hex

{
Servo Motor for DeltaRobot
PropForth5.2

Modified "jm_servo8_adv.spin"

12/10/2012 23:32:22
}

\ a cog special register
[ifndef ctra
h1F8	wconstant ctra
]

\ a cog special register
[ifndef frqa
h1FA	wconstant frqa 
]

\ a cog special register
[ifndef phsa
h1FC	wconstant phsa 
]

\ waitcnt ( n1 n2 -- n1 ) \ wait until n1, add n2 to n1
[ifndef waitcnt
: waitcnt
	_xasm2>1 h1F1 _cnip
;
]

d80 wconstant 1usec
d900 wconstant SV_MIN
d2000 wconstant SV_MAX

wvariable smin
wvariable smax
\ target[ch],delta[ch],pos[ch]: each channel is 2byte
wvariable target 4 allot
wvariable delta 4 allot
wvariable pos 4 allot


\ Set servo-motor's pin to output
\ ( n1 -- )  n1:first pin of contiguous servo group
: setup 3 0 do dup pinout 1+ loop drop ;

\ Check if position is inside region
\ ( n1 -- n2 )  
\ n1:servo position(microsecond)  
\ n2:SV_MIN <= n2 <= SV_MAX
: pos_check smin W@ max smax W@ min ;

\ Set selected servo (ch) to new position
\ ( n1 n2 n3 -- )  
\ n1:specified speed in us change per second (0 = no speed, direct move)
\ n2:ch
\ n3:newpos specified in microseconds
: setpos
pos_check                       
2dup
swap 2 u* target + W!
rot                             
dup 0 <=
if
     drop
     \ delta[ch] = 0
     over 2 u* delta + 0 swap W!
     \ pos[ch] = newpos
     swap 2 u* pos + W!     
else
     \ round up      (speed/d50) + (5/d10) = (speed + (5 X 5))/d50
     swap drop d25 + d50 u/ 1 max swap 2 u* delta + W! 
then
;

\ Move selected servo to newpos in ~ms milliseconds
\ ( n1 n2 n3 -- )  
\ n1:ms granularity is 20 (servo frame timing)
\ n2:ch
\ n3:newpos
\ not accurate at low ms values for long moves
: set_move
pos_check
rot d20 u/ dup 2 <
if
     drop swap 2dup
     \ save newpos to pos[ch] 
     2 u* pos + W!
     \ save newpos to target[ch]
     2 u* target + W!
else
     \ push ms/20
     >r 
     swap 2dup 
     2 u* pos + W@ - abs d10 u*
     \ pop ms/20
     r>
     \ calculate mdelta
     u/ 5 + d10 u/ 1 max
     
     over
     \ save mdelta to delta[ch]
     2 u* delta + W!
     \ save mdelta to target[ch]
     2 u* target + W!       
then
;

\ Returns position (current output command to servo) of channel
\ ( n1 -- n2 )  
\ n1:ch  n2:ch's position(microseconds)
: position 2 u* pos + W@ ;

\ Returns True when channel is at desired target
\ ( n1 -- n2 )  
\ n1:ch  n2:t/f
: at_target dup 2 u* pos + W@ swap 2 u* target + W@ = ;

\ Waits until servo channel reaches target position
\ ( n1 -- )  n1:ch
: wait begin dup at_target until drop ;

\ Runs up to 3 servos
\ (n1 -- )  
\ n1:first pin of contiguous servo group
\ runs in own cog; uses ctra
: delta_servo
dup
setup
1 frqa COG!
0 phsa COG!
cnt COG@ d200000 +            \ cnt + 2.5msec 
swap
begin
     \ run 8 slot(20msec)
     8 0 do                   
          i 3 <              
          if                  \ active channel?                          
               dup i + h10000000 or ctra COG!      \ PWM/NCO mode on servo pin
               i 2 u* pos + W@ 1usec u* negate phsa COG!
               i 2 u* pos + W@ i 2 u* target + W@ <
               if
                    i 2 u* pos + W@ i 2 u* delta + W@ + i 2 u* target + W@ min i 2 u* pos + W!
               else
                    i 2 u* pos + W@ i 2 u* target + W@ >
                    if
                         i 2 u* pos + W@ i 2 u* delta + W@ - i 2 u* target + W@ max i 2 u* pos + W!
          thens
          swap
          d200000 waitcnt         \ cnt + 2.5msec 
          swap                     
          0 ctra COG! 
     loop
0 until
\fkey? swap drop until
;

\ Start servo cog
\ ( n1 n2 -- )   
\ n1:posmax(microseconds)  n2:posmin(microseconds)  n3:first pin of contiguous servo group
: servo_init
SV_MIN smin W!
SV_MAX smax W!   
0 0 d940 setpos               \ servo1 initial position[0degree]      
0 1 d1000 setpos              \ servo2 initial position[0degree]
0 2 d970 setpos               \ servo3 initial position[0degree]
 c" 0 delta_servo" 0 cogx
;

 
decimal
